@@ -12,7 +12,7 @@ from django.contrib.gis.db.models import OneToOneField, ManyToManyField, Foreign
 from django.db.models import JSONField, AutoField, CharField, BooleanField, BigAutoField, DecimalField, \
     DateTimeField, DateField, BinaryField, TimeField, FloatField, EmailField, UUIDField, TextField, IntegerField, \
     BigIntegerField, NullBooleanField, Q
-from graphene import Scalar, InputObjectType, ObjectType, String
+from graphene import Scalar, InputObjectType, ObjectType, String, Field
 from graphql import parse
 from graphql.language import ast
 from graphql.language.printer import print_ast
@@ -21,7 +21,8 @@ from rescape_python_helpers import ramda as R, memoize
 from rescape_python_helpers.functional.ramda import to_dict_deep, flatten_dct_until, \
     to_array_if_not
 
-from .graphene_helpers import dump_graphql_keys, dump_graphql_data_object, camelize_graphql_data_object, call_if_lambda
+from .graphene_helpers import dump_graphql_keys, dump_graphql_data_object, camelize_graphql_data_object, call_if_lambda, \
+    resolve_field_type
 
 logger = logging.getLogger('rescape_graphene')
 from django.conf import settings
@@ -226,7 +227,7 @@ def input_type_class(field_config, crud, parent_type_classes, fields_only=False,
         parent_type_classes=modified_parent_type_classes,
         crud=crud,
         # Only continue with fields_only for search types. Otherwise we need types for the sub fields
-        fields_only=fields_only,
+        fields_only=create_filter_fields_for_search_type,
         with_filter_fields=with_filter_fields,
         create_filter_fields_for_search_type=create_filter_fields_for_search_type
     )
@@ -319,6 +320,7 @@ def fields_with_filter_fields(fields, graphene_class, parent_type_classes=[], cr
         # Keep our naming unique by appending parent classes, ordered newest to oldest
         R.concat([graphene_class], to_array_if_not(parent_type_classes)),
         fields_only=_fields_only,
+        with_filter_fields=with_filter_fields,
         create_filter_fields_for_search_type=create_filter_fields_for_search_type
     )
 
@@ -421,8 +423,8 @@ def process_field(field_to_unique_field_groups, field, field_dict_value, parent_
     :return: A dict with the unique property and anything else we need
     """
     unique = R.compact([
-        PRIMARY if R.has('primary_key', field) else None,
-        UNIQUE if R.has('unique', field) else None,
+        PRIMARY if R.prop_or(False, 'primary_key', field) else None,
+        UNIQUE if R.prop_or(False, 'unique', field) else None,
         R.prop_or(None, R.has('attname', field), field_to_unique_field_groups)
     ])
     # Normally the field_dict_value will delegate the type to the underlying Django model
@@ -509,15 +511,31 @@ def merge_with_django_properties(graphene_type, field_dict):
 
 @R.curry
 def resolve_type(graphene_type, field_config, fields_only=False):
-    # If the type is a scalar, just instantiate
-    # Otherwise created a related field InputType subclass. In order to query a nested object, it has to
-    # be an input field. Example: If A User has a Group, we can query for users named 'Peter' who are admins:
-    # graphql: users: (name: "Peter", group: {role: "admin"})
-    # https://github.com/graphql-python/graphene/issues/431
-    # When fields_only=False, meaning we have an input_type_class, instantiate it with ()
-    if inspect.isclass(R.prop('type', field_config)) and issubclass(R.prop('type', field_config), Scalar):
+    """
+     Resolves the field type, instantiating scalars and recursing on input types (complex types)
+     If fields_only is true (default Fasle), just return the field_config for scalars without instantiating
+     and for complex types pass fields_only to the recursion
+     If the type is a scalar, just instantiate
+     Otherwise created a related field InputType subclass. In order to query a nested object, it has to
+     be an input field. Example: If A User has a Group, we can query for users named 'Peter' who are admins:
+     graphql: users: (name: "Peter", group: {role: "admin"})
+     https://github.com/graphql-python/graphene/issues/431
+     When fields_only=False, meaning we have an input_type_class, instantiate it with ()
+
+    :param graphene_type: The graphene type of the class containing the field_config. Used only for naming
+    the parent class in recursion
+    :param field_config: Contains either type (coming from a Django model type definition) or graphene type
+    (coming from a non-Django model type definition). This is the type that is optionally instnatiated or
+    recursed upon
+    :param fields_only: Default False, only get the field config for scalars and recurse with this flag for
+    complex types
+    :return: The input type of the field config or the one of those as the result of recursion
+    """
+
+    field_type = resolve_field_type(field_config)
+    if inspect.isclass(field_type) and issubclass(field_type, Scalar):
         # Return the type instantiated
-        return R.prop('type', field_config)() if not fields_only else field_config
+        return field_type() if not fields_only else field_config
     else:
         # Resolve with recursion
         input_type_class_instance = input_type_class(field_config, READ, graphene_type, fields_only=fields_only)
@@ -556,7 +574,8 @@ def allowed_filter_pairs(field_name, graphene_instance, field_config, fields_onl
     """
 
     def filter_field_config_or_type(config):
-        type_modifier = config['type_modifier'] if R.has('type_modifier', config) else lambda t: t()
+        type_modifier = config['type_modifier'] if R.has('type_modifier', config) else lambda *type_and_args: \
+            type_and_args[0]()
         return R.merge(field_config, dict(type_modifier=type_modifier)) if fields_only else (
             # If a type_modifier is needed for the filter type, such as a List constructor call it
             # with the field's type as an argument
@@ -573,7 +592,8 @@ def allowed_filter_pairs(field_name, graphene_instance, field_config, fields_onl
         R.filter_dict(
             lambda keyvalue: field_name not in EXCLUDED_PROP_KEYS_FROM_FILTERING and (
                     not R.has('allowed_types', keyvalue[1]) or
-                    R.any_satisfy(lambda typ: issubclass(graphene_instance.__class__, typ), keyvalue[1]['allowed_types'])
+                    R.any_satisfy(lambda typ: issubclass(graphene_instance.__class__, typ),
+                                  keyvalue[1]['allowed_types'])
             ),
             FILTER_FIELDS
         )
@@ -627,11 +647,11 @@ def top_level_allowed_filter_arguments(fields, graphene_type, with_filter_fields
     do want it recursively on the objects property
     :return: dict of field keys and there graphene type, either a primitive or input type
     """
-    return type_modify_fields(input_type_class(
+    return input_type_class(
         dict(fields=fields, graphene_type=graphene_type),
         'read', [], fields_only=True, with_filter_fields=with_filter_fields,
         create_filter_fields_for_search_type=create_filter_fields_for_search_type,
-    ))
+    )
 
 
 def allowed_filter_arguments(fields_dict, graphene_type, fields_only=False):
@@ -676,7 +696,9 @@ def guess_update_or_create(fields_dict):
 
 
 def instantiate_graphene_type_or_fields(field_config, parent_type_classes, crud,
+                                        field_name,
                                         fields_only=False,
+                                        with_filter_fields=True,
                                         create_filter_fields_for_search_type=False):
     """
         Instantiates the Graphene type at value.type. Most of the time the type is a primitive and
@@ -687,6 +709,7 @@ def instantiate_graphene_type_or_fields(field_config, parent_type_classes, crud,
     :param fields_only Only create fields, not the type
     :param parent_type_classes: String array of parent graphene types for dynamic class naming
     :param crud: READ, WRITE, UPDATE, DELETE or None if create_filter_fields_for_search_type is true
+    :param field_name The field name, just for debugging
     :param create_filter_fields_for_search_type Default False, Usually we only add filter fields for READ crud types. This
     overrides that so that Search types can add filters
     :return:
@@ -709,7 +732,7 @@ def instantiate_graphene_type_or_fields(field_config, parent_type_classes, crud,
             crud,
             parent_type_classes,
             fields_only=fields_only,
-            with_filter_fields=True,
+            with_filter_fields=with_filter_fields,
             create_filter_fields_for_search_type=create_filter_fields_for_search_type
         )
     elif R.isfunction(graphene_type) and \
@@ -746,22 +769,27 @@ def instantiate_graphene_type_or_fields(field_config, parent_type_classes, crud,
 
 
 def input_type_fields(fields_dict, crud, parent_type_classes=[], fields_only=False,
+                      with_filter_fields=True,
                       create_filter_fields_for_search_type=False):
     """
     :param fields_dict: The fields_dict for the Django model or json data
     :param crud: INSERT, UPDATE, or DELETE. If None the type is guessed
     :param parent_type_classes: String array of parent graphene types for dynamic class naming
     :param fields_only
+    :param with_filter_fields Default True, indicates to add filter fields for queries. create_filter_fields_for_search_type
+    is used only for SearchTypes that have filter fields as type fields
     :param create_filter_fields_for_search_type: Default false. Only used by Search types to add filter versions of their fields
     :return:
     """
     # Don't guess the crud type if create_filter_fields_for_search_type is true. We want it null in that case
     crud = crud or (guess_update_or_create(fields_dict) if not create_filter_fields_for_search_type else crud)
-    return R.map_dict(
-        lambda field_config: instantiate_graphene_type_or_fields(
-            field_config, parent_type_classes, crud,
+    return R.map_with_obj(
+        lambda field_name, field_config: instantiate_graphene_type_or_fields(
+            # field_name is just passed for debugging
+            field_config, parent_type_classes, crud, field_name,
             fields_only=fields_only,
-            create_filter_fields_for_search_type=create_filter_fields_for_search_type
+            with_filter_fields=with_filter_fields,
+            create_filter_fields_for_search_type=create_filter_fields_for_search_type,
         ),
         # Filter out values that are deny
         # This means that if the user tries to pass these fields to graphql an error will occur
@@ -873,7 +901,12 @@ def graphql_query(graphene_type, fields, query_name):
     The only key allowed is variables, which contains param key values. Example: variables={'user': 'Peter'}
     This results in query whatever(id: String!) { query_name(id: id) ... }
     """
-    field_type_lookup = top_level_allowed_filter_arguments(fields, graphene_type)
+
+    # If we already have a search class, it has filters as fields, so don't add them here
+    field_type_lookup = top_level_allowed_filter_arguments(fields, graphene_type) if\
+        'Search' not in graphene_type.__name__ else\
+        type_modify_fields(fields)
+
 
     def form_query(client, field_overrides={}, **kwargs):
         """
@@ -1224,7 +1257,21 @@ def update_or_create_with_revision(model_class, update_or_create_values):
 
     # Declare a revision block.
     with reversion.create_revision():
-        return model_class.objects.update_or_create(**update_or_create_values)
+        if not R.prop_or(
+                False,
+                'id',
+                update_or_create_values
+        ) and not R.prop_or(
+            False,
+            'defaults',
+            update_or_create_values):
+            # If there is no id and no defaults, we have to do a straight save.
+            # update_or_create could match multiple existing instances
+            instance = model_class(**update_or_create_values)
+            instance.save()
+            return instance, True
+        else:
+            return model_class.objects.update_or_create(**update_or_create_values)
 
 
 def deep_merge_existing_json(django_model, json_prop, data):
@@ -1391,8 +1438,27 @@ def apply_type(v):
 
     # If we have allowed arguments make args a 2 element array. The first element is always the graphene type to
     # construct
-    args = [R.prop('type', v)] + ([allowed_arguments] if allowed_arguments else [])
-    t = R.prop_or(lambda typ: typ(), 'type_modifier', v)
+    args = [R.if_else(
+        R.has('type'),
+        R.prop('type'),
+        R.prop('graphene_type')
+    )(v)] + ([allowed_arguments] if allowed_arguments else [])
+
+    def x(typ, allowed_args):
+        # TODO there seems to be a case where using v['type'] on fields of DjangoTypes doesn't make sense.
+        # v['type'] is actually an input type, so it doesn't make sense for top-level declarations.
+        # In this case just construct the graphene type with allow args
+        return Field(v['graphene_type'], allowed_args)
+
+    def y(typ):
+        return typ()
+
+    t = R.prop_or(
+        x if R.length(args) == 2 else y,
+        'type_modifier',
+        v
+    )
+
     return t(*args)
 
 
